/* mx.c  exemple xlib - initialisation fentre
 * fonctions de base dessin Xlib et gestion vnements
 * compil : sur linux et solaris : gcc -c mx.c
 * utilisation : gcc -o prog prog.c mx.o -L/usr/X11R6/lib -lX11
 */
#include <stdio.h>	/* pour printf() */
#include <stdlib.h>	/* pour EXIT_FAILURE */
#include <X11/Xlib.h>
#include <X11/Xutil.h>	/* : #define IsFunctionKey(keysym) */
#include <X11/keysym.h> /* idem (include keysymdef.h) */

#define red     0
#define green   1
#define blue    2
#define yellow  3
#define white   4
#define black   5

char *couleur[9] = {	"red","green","blue",
			"yellow","white","black",
			"purple","cyan","gold"	};
int dbg = 0; /* debug level */

/*--- structures de donnes pour l'utilisation de X11 ---*/
        Display *xdisplay;
        Window xwindow;
        int xscreen;
        unsigned long xforeground, xbackground;
        XEvent xevent;
        GC xgc;
        Colormap cmap;
        XColor color, colorrgb;

/* ---------------------- fonctions de dessin Xlib ---------------- */

drawstr (int x, int y, char *str, int lng, int col)
{
/* Draw string at (x,y) in col */
if (XAllocNamedColor (xdisplay, cmap, couleur[col], &color, &colorrgb))
    XSetForeground (xdisplay, xgc, color.pixel);
XDrawString (xdisplay, xwindow, xgc, x, y, str, lng);
}

drawrec (int x, int y, int larg, int hter, int col)
{
/* Draw rect at (x,y) in col */
if (XAllocNamedColor (xdisplay, cmap, couleur[col], &color, &colorrgb))
    XSetForeground (xdisplay, xgc, color.pixel);
XDrawRectangle (xdisplay, xwindow, xgc, x, y, larg, hter);
}

drawvec (int x1, int y1, int x2, int y2, int col)
{
/* Draw vector in col */
if (XAllocNamedColor (xdisplay, cmap, couleur[col], &color, &colorrgb))
    XSetForeground (xdisplay, xgc, color.pixel);
XDrawLine (xdisplay, xwindow, xgc, x1, y1, x2, y2);
}

drawarc (int x, int y, int larg, int hter, int ang1, int ang2, int col)
{
/* Draw arc at (x,y) in white */
if (XAllocNamedColor (xdisplay, cmap, couleur[col], &color, &colorrgb))
    XSetForeground (xdisplay, xgc, color.pixel);
XDrawArc (xdisplay, xwindow, xgc, x, y, larg, hter, ang1, ang2);
}

fillrec (int x, int y, int larg, int hter, int col)
{
/* Fill rectangle at (x,y), width , height, */
if (XAllocNamedColor (xdisplay, cmap, couleur[col], &color, &colorrgb))
   XSetForeground (xdisplay, xgc, color.pixel);
XFillRectangle (xdisplay, xwindow, xgc, x, y, larg, hter);
}

fillcerc (int x, int y, int larg, int hter, int col)
{ /* Fill cercle at (x,y), width , height, with col */
if (XAllocNamedColor (xdisplay, cmap, couleur[col], &color, &colorrgb))
   XSetForeground (xdisplay, xgc, color.pixel);
XFillArc (xdisplay, xwindow, xgc, x, y, larg, hter, 0, 360*64);
}

flushfx ()
{    XFlush (xdisplay); /* flush X request queue to server */
}

detruitfx()
{
   XDestroyWindow(xdisplay, xwindow);
   XCloseDisplay (xdisplay);
}


/* --------------------- initialisation de la Xlib ---------------- */

initfx(int width, int height)
{
        xdisplay = XOpenDisplay ("");  /* connect to the X server */
        if (xdisplay == NULL) {
                fprintf (stderr, "cannot connect to server\n");
                exit (EXIT_FAILURE); }

        xscreen = DefaultScreen (xdisplay); /* get default screen */

        /* get black and white representation on current screen */        
        xbackground = BlackPixel (xdisplay, xscreen);
        xforeground = WhitePixel (xdisplay, xscreen);

        /* Create window at (ix,iy), width 450, height 250, 
           border width 2, in default root  */
        xwindow = XCreateSimpleWindow (xdisplay,
                DefaultRootWindow(xdisplay), 10, 10, width, height, 2,
                xforeground, xbackground);
        if (xwindow == 0) {
                fprintf (stderr, "cannot open window\n");
                exit (EXIT_FAILURE); }

        /* ask for exposure event */
	/* liste des masques et events dans /usr/X11R6/include/X11/X.h */

        XSelectInput(xdisplay, xwindow, 
        ButtonPressMask | ExposureMask | ButtonMotionMask | \
 	ButtonReleaseMask ); 

        /* pop this window up on the screen */
        XMapRaised (xdisplay, xwindow);

        /* wait for the window showing up before continuing */
        XNextEvent (xdisplay, &xevent);

        /* set graphics context of main rectangle */
        xgc= XCreateGC (xdisplay, xwindow, 0, 0);

        if (DisplayPlanes (xdisplay, xscreen) != 1) {
            cmap = DefaultColormap (xdisplay, xscreen);
        }
	return 0;
}

/* -------------------------- gestion vnements ------------------ */

int nextevent (int *event, int *x, int *y, int *keysym, char *c)
/* event, x,y, keysym,c sont des sorties de nextevent */
/* appel nextevent synchrone : on reste dans nexevent jusqu' ce
   qu'un vnement se produire.
*/
{
	/* data pour gestion frappes clavier */
	KeySym ksym, ksym1;
	char *ksymname;
	char buf[10];
	int lbuf=10;
	XComposeStatus istat;
	
while (1) {	/* wait for next event */
	XNextEvent (xdisplay, &xevent);
	if (dbg==2) printf("-xevent.type=%d\n",xevent.type);
	buf[0]= '\0'; buf[1]= '\0';
	*x = 0; *y = 0; *keysym = 0; *c = '\0';
	*event = xevent.type;
	
  switch (xevent.type) {
	case Expose:
        	return 0;

	case MotionNotify:
		*x = xevent.xbutton.x;
		*y = xevent.xbutton.y;
        	return 0;
		
	case ButtonPress:
		*x = xevent.xbutton.x;
		*y = xevent.xbutton.y;
		return 0;
		
	case ButtonRelease:
		*x = xevent.xbutton.x;
		*y = xevent.xbutton.y;
		return 0;
		
  } /* fin switch */
} /* fin while(1) */
} /* fin nextevent */

/* ------------------------------- fin mx.c ----------------------- */
